### event

Элемент реализует поддержку дополнительных типов событий jQuery. Дополнительные типы реализованы в виде значений модификатора `type`.

## Модификаторы элемента

### type

Модификатор предоставляет набор полифилов, реализующих уровень абстракции над `jquery`-событиями устройств ввода. Это позволяет создавать общую логику для различных платформ (десктопы, телефоны и т.д.) и дополнять ее методами, специфическими для конкретного вида устройств.

Каждый полифил добавляет некоторый набор **pointer-событий**, который может быть использован для создания аппаратно независимой логики. 

В зависимости от значения модификатора, при его установке будут подключены разные полифилы. Доступен следующий набор значений:

* `type_pointer` – модификатор для подключения всех типов pointer-событий. Не вводит дополнительной логики.
* `type_pointerclick` – полифил, реализующий событие `pointerclick`. Использование полифила позволяет избавиться от задержки события `click` на тач-устройствах.
* `type_pointernative` – полифил, реализующий базовую функциональность модели [W3C Pointer Events](http://www.w3.org/TR/pointerevents/). Предоставляет следующий набор событий:
    * `pointerover` – событие создается в момент, когда указатель находится над элементом. Для устройств, не поддерживающих `hover`, событие создается перед событием `pointerdown`.
    * `pointerenter` – вход указателя в активную зону элемента. Событие создается, когда указатель находится над элементом или одним из его потомков, в том числе, по событию `pointerdown` для устройств, не поддерживающих `hover`. Событие аналогично `pointerover`, но не всплывает.
    * `pointerdown` – событие создается в момент, когда устройство ввода входит в состояние **активного нажатия**. Для мыши это соответствует состоянию, когда хотя бы одна кнопка нажата, для перьевых и тач- устройств – состоянию контакта с поверхностью дигитайзера.
    * `pointermove` – событие создается при изменении координат указателя. 
    * `pointerup` – выходит из состояния активного нажатия. Для мыши это соответствует переходу от состоянию, когда была нажата хотя бы одна кнопка, к состоянию без нажатых кнопок. Для перьевых и тач- устройств – моменту разрыва контакта с поверхностью дигитайзера.
    * `pointerout` – событие создается в момент:
        *  когда указатель покидает зону над элементом;
        *  после события `pointerup` для устройств, не поддерживающих `hover`; 
        *  после события `pointercancel`;
        *  когда стилус покидает рабочую зону дигитайзера.
    * `pointerleave` – выход указателя из активной зоны элемента. Событие создается, когда указатель покидает зону над элементом или одним из его потомков, в том числе по событиям `pointerup` и `pointercancel` для устройств, не поддерживающих `hover`. Событие аналогично `pointerout`, но не всплывает.
* `type_pointerpressrealease` – подключает полифил, реализующий события `pointerpress` и `pointerrelease`. Полифил использует события Pointer Events:
    * `pointerpress` – создается по событию `pointerdown`.
    * `pointerrelease` – создается по событиям `pointerup` и `pointercancel`.

Все pointer-события являются пользовательскими событиями jQuery. Подписка на pointer-события осуществляется стандартным образом:

```js
modules.define(
    'pointer-test',
    ['i-bem__dom', 'jquery__event_type_pointer'],
    function(provide, BEMDOM, $) {

provide(BEMDOM.decl({ block : this.name }, /** @lends pointer-test.prototype */
{ 
    onSetMod : {
        'js' : {
            'inited' : function() {
                this.bindTo('pointerpress', this.onPress); // при инициализации подписываемся на pointerpress на самом блоке и передаем обработчик
            }   
        }

    },
    onPress : function(e) {
        console.log(e.type);
        this.bindTo('pointerrelease', this.onRelease); // при вызове обработчика подписываемся на pointerrelease и передаем его обработчик
    },
    onRelease : function(e) {
        console.log(e.type);
        this.unbindFrom('pointerrelease', this.onRelease); // при вызове отписываемся от pointerrelease
    }
}));
});
```
