# objects

Блок `objects` предоставляет набор методов-хелперов для работы с объектами JavaScript.

Блок реализован в технологии `vanilla.js` и подходит для использования как на сервере, так и на клиенте.

## Публичные методы блока

### `extend` 

Метод позволяет расширять объект свойствами другого объекта. Копируются только собственные свойства объекта, не полученные по цепочке прототипов. Метод принимает аргументами:

* `target` `{Object}` – целевой объект, который расширяется свойствами других.
* `source` `{Object}` – объект, свойства которого добавляются к целевому. Может быть передано несколько объектов. Свойства каждого из них будут добавлены к целевому.

Метод возвращает целевой объект с добавленными свойствами.

Например:

```js
modules.require(['objects'], function(objects) {
    var obj1 = { a : 1, b : 2 },
        obj2 = { b : 3, c : 4 };

    console.log(objects.extend(obj1, obj2)); // { a : 1, b : 3, c : 4 }
});
```


### `isEmpty`

Метод позволяет выяснить пуст ли переданный ему объект. Другими словами, имеет ли объект собственные свойства.

Метод принимает один аргумент `{Object}` – объект для проверки. Возвращаемое значение `{Boolean}` – `true` в случае, если объект не имеет собственных свойств.


```js
modules.require(['objects'], function(objects) {
    var obj1 = {},
        obj2 = { foo : 'bar' };

    console.log(objects.isEmpty(obj1)); // true
    console.log(objects.isEmpty(obj2)); // false
});
```

### `each`

Метод служит для итеративного обхода собственных свойств объекта. Для каждого собственного свойства вызывается функция-callback. Метод не возвращает значения и принимает следующие аргументы:

* `obj` `{Object}` – объект обход свойств которого производится.
* `fn` `{Function}` – callback, вызываемый для каждого свойства.
* `ctx` `{Object}` – контекст callback-функции. Опционально.

Первым аргументом callback получает значение свойства объекта, для которого была вызвана, вторым имя свойства.

```js
modules.require(['objects'], function(objects) {
    var obj1 = { a : 1, b : 2, c : 4 },
        logValue = function(val) {
            console.log(val);
        };

    objects.each(obj1, logValue); // 124
});
```


Примеры использования `objects` в библиотеке `bem-components`:

* блок `popup` [с модификатором target](https://github.com/bem/bem-components/blob/v2/common.blocks/popup/_target/popup_target.js#L82).
