﻿# `page`

Блок `page` создаёт HTML-теги верхнего уровня страницы:

* `<html>`
* `<head>`
* `<body>`

Он отвечает за подключение к странице CSS и JS, выставление тегов `meta` и указание заголовка. Для этого в BEMJSON-декларации блока и элементов блока зарезервированы специальные свойства.

## Свойства блока

### `title`

Минимальная декларация блока предполагает указание свойства `title`, которое превращается в HTML-тег `<title>`.

```js
{
    block: 'page',
    title: 'Page title',
    content: 'Блок page'
}
```


### `doctype`

Позволяет явно указать DTD (document type definition) тип текущего документа. Если свойство не задано, по умолчанию будет использоваться `<!DOCTYPE html>`.

### `favicon`

Позволяет указать URL значка страницы (фавиконки). На странице строка с URL становится HTML-атрибутом `href`:

```js
{
    block: 'page',
    title: 'Page title',
    favicon: 'favicon.ico',
    content: 'Страница с пользовательской фавиконкой'
}
```


### `head`

Свойство `head` дополняет содержимое `HTML`-тега `<head>`, определенное в шаблоне блока:

```js
{
    block: 'page',
    title: 'Page title',
    head: [
        { elem: 'css', url: 'example.css' },
        { elem: 'js', url: 'example.js' },
        { elem: 'meta', attrs: { name: 'description', content: 'Yet another webdev blog' } }
    ],
    content: 'Страница с подключенными CSS, JS и meta-данными'
}
```


### `styles`

Предоставляет альтернативный способ подключения таблиц стилей CSS. Значением служит массив хешей подключаемых элементов `css`:

```js
{
    block: 'page',
    title: 'Page title',
    styles: [
        { elem: 'css', url: '_index.css' }
    ],
    content: 'Страница с подключенным CSS'
}
```


Подключение стилей с помощью свойств `styles` и `head` производится идентично. 

### `scripts`

Используется для подключения сценариев JS в тело страницы. Для каждого подключаемого сценария внутри HTML-тега `<body>` (в конце) создается тег `<script>`. Значением свойства служит массив хешей подключаемых элементов `js`:

```js
{
    block: 'page',
    title: 'Page title',
    scripts: [
        { elem: 'js', url: '_index.js' }
    ],
    content: 'Страница со скриптом подключенным в body'
}
```


### `content`

Значением свойства `content` блока `page` может быть хеш (если блок единственный) или массив хешей блоков – содержимое страницы:

```js
{
    block: 'page',
    title: 'Page title',
    content: {
        block: 'link',
        mods: { pseudo: 'yes', togcolor: 'yes', color: 'green' },
        url: '#',
        target: '_blank',
        title: 'Кликни меня',
        content: 'Псевдоссылка, меняющая цвет по клику'
    }
}
```


К блокам, содержащимся в свойстве `content`, применяются их шаблоны.

## Элементы блока

### `css` 

Элемент `css` служит для подключения таблиц стилей CSS или передачи CSS-свойств. В зависимости от того, указано ли в декларации элемента поле `url`, создается HTML-элемент с тегом:

* `<link>` и свойством `stylesheet`, если поле `url` указано. Свойство `href` элемента получает ссылку из `url` BEMJSON-декларации.
* `<style>` если поле `url` неуказано. В этом случае предполагается, что содержимое тега передается с помощью свойства `content` BEMJSON-декларации элемента.

#### Свойства элемента

##### `content` 

Служит для явной передачи содержимого тега `<style>`:

```js
{
    block: 'page',
    title: 'Page title',
    head: [
        {
            elem: 'css',
            content: '.page { color: #f00 }'
        }
    ],
    content: 'Страница с тэгом <style>'
}
```


### `js` 

Элемент `js` действует аналогично `css`, подключая к странице сценарии JavaScript при помощи тега `<script>`. Свойство `src` тега получает ссылку, указанную в поле `url` декларации элемента.

### `meta`

Элемент `meta` служит для создания HTML-элементов `<meta>` и указания для них пользовательских метаданных. Метаданные передаются как ключи и значения хеша атрибутов – свойства `attrs` BEMJSON-декларации элемента:

```js
{
    block: 'page',
    title: 'Page title',
    head: [
        { elem: 'css', url: 'example.css' },
        { elem: 'meta', attrs: { name: 'keywords', content: 'js, css, html' } }
    ],
    content: 'Страница с подключенным CSS и meta-данными'
}
```


Пример наиболее распространенных атрибутов для `meta`:

* `charset` – задает кодировку используемую на странице. По умолчанию блоком на уровне шаблона создается элемент `meta` со свойством `charset` в значении `utf-8`.
* `content` – позволяет задавать содержимое, ассоциируемое с атрибутами `http-equiv` или `name`.
* `http-equiv` – значение атрибута, заданное в `content`, влияет на поведение браузера: выбор формат заголовка HTTP, кодировку, срок устаревания документа.
* `name` – задает тип метаданных, указанных в атрибуте `content`. Не следует использовать атрибут одновременно с атрибутами `http-equiv` или `charset`. Наиболее вероятными типами могут быть:
    * `application-name` – позволяет указать имя веб-приложения доступного на странице.
    * `author` – имя автора документа в произвольном формате.
    * `description` – краткое описание страницы. Используется некоторыми браузерами при создании закладки.
    * `keywords` – список ключевых слов, характеризующих страницу. Передается в виде строки, разделенной запятыми.
    * `robots` – задает инструкции по индексированию страницы для дружественных поисковых роботов. Инструкции передаются в виде строки, разделенной запятыми. 

```js
{
    block: 'page',
    title: 'Page title',
    head: [
        { elem: 'css', url: 'example.css' },
        { elem: 'js', url: 'example.js' },
        { elem: 'meta', attrs: { name: 'robots', content: 'index, nofollow' } },
        { elem: 'meta', attrs: { name: 'description', content: 'Yet another webdev blog' } }
    ],
    content: 'Страница с подключенными CSS, JS и meta-данными'
}
```


#### Указание кодировки с помощью `meta`  

```js
{
    block: 'page',
    title: 'Page title',
    head: [
        { elem: 'meta', attrs: { 'http-equiv': 'content-type', content: 'text/html; charset=utf-8' } }
    ],
    content: 'Страница в utf-8'
}
```


#### использование IE9 (и выше) в режиме максимальной совместимости

```js
{
    block: 'page',
    title: 'Page title',
    head: [
        { elem: 'meta', attrs: { 'http-equiv': 'X-UA-Compatible', content: 'IE=EmulateIE7, IE=edge' } }
    ],
    content: 'Страница в режиме эмуляции IE7'
}
```


## Дополнительные варианты использования блока

### Выставление значения тега `<title>` из контекста

`BEMHTML`:

```js
content: [
    // ...
    {
        tag: 'title',
        content: this.ctx.title
    },
    // ...
]
```


### Выставление фавиконки из контекста

`BEMHTML`:

```js
content: [
    // ...
    this.ctx.favicon ? {
        elem: 'favicon',
        url: this.ctx.favicon
    } : '',
    // ...
]
```


### Отмена автоматической инициализации блоков

Для отмены автоматической инициализации блоков страницы нужно создать на уровне проекта файл `page.deps.js`, содержащий следующий код:

```js
noDeps: [
    { block: 'i-bem', elem: 'dom', mods: { init: 'auto' } }
]
```
