# ua

Блок служит для сбора данных о мобильном браузере пользователя и позволяет определить:

* мобильную платформу;
* типа браузера;
* версию браузера;
* тип соединения;
* наличие поддержки видео и SVG;
* поддержку технологии Adobe Flash;
* ориентацию и размер экрана;
* соотношение сторон у экрана устройства.

Блок реализован в технологии JS и подходит для использования в клиентских приложениях.

```js
modules.define('uatest', ['i-bem__dom', 'ua'], function(provide, BEMDOM, ua) {
provide(BEMDOM.decl(this.name, /** @lends uatest.prototype */
{ 
    onSetMod : {
        'js' : {
            'inited' : function() {
                console.dir(ua);
            }   
        }
    }
}));
});
```


Объект, получаемый callback-функцией после подключения блока, содержит набор свойств, характеризующих:

* `ua` `{String}` – тип браузера.
* версию мобильной платформы(`{String|undefined}`). Строка с текущей версией содержится в поле, соответствующем типу платформы:
    * `ios` – версия iOS.
    * `android` – версия Android.
    * `bada` – версия Bada Os.
    * `wp` – версия Windows Phone.
    * `other` `{Boolean}` – мобильная платформа неопределена. Устанавливается в значение `true` для всех мобильных платформ, кроме вышеперечисленных.
* версию браузера (`{String}`). Доступно определение версий для следующих типов браузеров:
    * Opera – свойство `opera`.
    * Chrome – свойство `chrome`.
* `iphone` `{Boolean}` – устройство как iPhone.
* `ipad` `{Boolean}` – устройство как iPad.
* `screenSize` `{String}` – размер экрана. Доступны следующие значения:
    * large – размер экрана больше 320 px.
    * normal – размер экрана равен 320 px.
    * small – размер экрана меньше 320 px.
* тип соединения(`{String}`). Доступны следующие значения:
    * `wifi` – соединение с помощью Wi-Fi.
    * `3g` – соединение с помощью 3G.
    * `2g` – соединение с помощью EDGE и GSM.
* `dpr` `{Number}` – относительную плотность пикселей. Коэффициент характеризует отношение физических пикселей устройства к аппаратно независимым (dppx). Позволяет определить не использует ли устройство дисплей с повышенной плотностью пикселей (например, Retina). По умолчанию равен `1`.
* `flash` `{String|undefined}` –  версию Adobe Flash, установленную в браузере.
* `video` `{Boolean}` – поддержку видео. Имеет значение `true`, если видео поддерживается.
* `svg` `{Boolean}` – поддержку формата SVG. Имеет значение `true`, если SVG поддерживается.
* `width` `{Number}` – ширину рабочей области экрана в px.
* `height` `{Number}` – высоту рабочей области экрана в px.
* `landscape` `{Boolean}` – ориентацию экрана. Имеет значение `true` при горизонтальной ориентации.

Например, можно проверить не использует ли устройство дисплей с повышенной плотностью пикселей и, если использует, отдавать браузеру изображения с высоким разрешением: 

```js
modules.define('image', ['i-bem__dom', 'ua'], function(provide, BEMDOM, ua) {
provide(BEMDOM.decl(this.name, {
    onSetMod: {
        js: {
            inited: function() {
                // если модификатор hi не установлен 
                // и относительная плотность выше 1 устанавливаем hi
                !this.hasMod('hi') && 
                    ua.dpr > 1 && 
                        this.setMod('hi');
            }
        },
        'hi': function() {
            var _oldUrl = this.domElem.attr('src');
            // подменяем путь к изображению в HTML-атрибуте src
            this.domElem.attr('src', _oldUrl.replace('images/', 'images/hi/'));
        }            
    }
}));
});
```
